import { useState, useRef, useMemo } from 'react';
import { DebouncedInput } from './DebouncedInput.js';
import { Button, Icon, Popover } from '@equinor/eds-core-react';
import { filter_alt_active, filter_alt } from '@equinor/eds-icons';
import styled from 'styled-components';
import { tokens } from '@equinor/eds-tokens';
import { jsxs, Fragment, jsx } from 'react/jsx-runtime';

/* istanbul ignore file */

const NumberContainer = styled.div.withConfig({
  displayName: "Filter__NumberContainer",
  componentId: "sc-ytpdpw-0"
})(["display:grid;grid-template-columns:80px 80px;grid-column-gap:32px;"]);
function Filter({
  column,
  table
}) {
  const firstValue = table.getPreFilteredRowModel().flatRows[0]?.getValue(column.id);
  const [open, setOpen] = useState(false);
  const filterIconRef = useRef();
  const togglePopover = event => {
    event.stopPropagation();
    setOpen(!open);
  };
  const columnText = useMemo(() => {
    let header;
    try {
      if (typeof column.columnDef.header === 'function') {
        const obj = column.columnDef.header(
        // eslint-disable-next-line @typescript-eslint/no-explicit-any,@typescript-eslint/no-unsafe-argument
        {});
        header = obj.props.children;
      } else {
        header = column.columnDef.header;
      }
    } catch {
      /*em all*/
    }
    return header;
  }, [column.columnDef]);
  const columnFilterValue = column.getFilterValue();
  const sortedUniqueValues = useMemo(() => typeof firstValue === 'number' ? [] : Array.from(column.getFacetedUniqueValues().keys()).sort().map(v => v ?? 'NULL_OR_UNDEFINED'),
  // eslint-disable-next-line react-hooks/exhaustive-deps
  [column.getFacetedUniqueValues()]);
  const hasActiveFilters = value => {
    if (Array.isArray(value)) {
      if (typeof firstValue === 'number') {
        return value.some(v => !isNaN(v) && !!v);
      } else {
        return value.filter(v => !!v).length > 0;
      }
    }
    return value;
  };
  return /*#__PURE__*/jsxs(Fragment, {
    children: [/*#__PURE__*/jsx(Button, {
      "aria-haspopup": true,
      "aria-expanded": open,
      "data-testid": 'open-filters',
      ref: filterIconRef,
      onClick: togglePopover,
      variant: 'ghost_icon',
      children: /*#__PURE__*/jsx(Icon, {
        color: tokens.colors.text.static_icons__default.hex,
        data: hasActiveFilters(columnFilterValue) ? filter_alt_active : filter_alt
      })
    }), /*#__PURE__*/jsx(Popover, {
      style: {
        width: typeof firstValue === 'number' ? '220px' : '340px'
      },
      anchorEl: filterIconRef.current,
      open: open,
      onClose: () => setOpen(false),
      children: /*#__PURE__*/jsx(Popover.Content, {
        style: {
          width: typeof firstValue === 'number' ? '180px' : '310px'
        },
        children: typeof firstValue === 'number' ? /*#__PURE__*/jsxs(NumberContainer, {
          children: [/*#__PURE__*/jsx(DebouncedInput, {
            type: "number",
            values: sortedUniqueValues,
            min: Number(column.getFacetedMinMaxValues()?.[0] ?? ''),
            max: Number(column.getFacetedMinMaxValues()?.[1] ?? ''),
            value: columnFilterValue?.[0] ?? '',
            onChange: value => column.setFilterValue(old => [value, old?.[1]]),
            placeholder: `Min ${column.getFacetedMinMaxValues()?.[0] ? `(${column.getFacetedMinMaxValues()?.[0]})` : ''}`
          }), /*#__PURE__*/jsx(DebouncedInput, {
            type: "number",
            values: sortedUniqueValues,
            min: Number(column.getFacetedMinMaxValues()?.[0] ?? ''),
            max: Number(column.getFacetedMinMaxValues()?.[1] ?? ''),
            value: columnFilterValue?.[1] ?? '',
            onChange: value => column.setFilterValue(old => [old?.[0], value]),
            placeholder: `Max ${column.getFacetedMinMaxValues()?.[1] ? `(${column.getFacetedMinMaxValues()?.[1]})` : ''}`
          })]
        }) : /*#__PURE__*/jsx(DebouncedInput, {
          type: "text",
          label: columnText,
          values: sortedUniqueValues,
          debounce: 100,
          value: columnFilterValue ?? [],
          onChange: value => column.setFilterValue(value),
          placeholder: `${(columnFilterValue ?? []).length} / ${column.getFacetedUniqueValues().size} selected`,
          list: column.id + 'list'
        })
      })
    })]
  });
}

export { Filter };
