import { flexRender } from '@tanstack/react-table';
import { Table, Icon } from '@equinor/eds-core-react';
import { arrow_up, arrow_down } from '@equinor/eds-icons';
import { useTableContext } from '../EdsDataGridContext.js';
import { Filter } from './Filter.js';
import styled from 'styled-components';
import { tokens } from '@equinor/eds-tokens';
import { useMemo } from 'react';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';

const getSortLabel = sorted => {
  if (sorted) {
    return `${sorted}ending`;
  }
  return 'none';
};
const ResizeInner = styled.div.withConfig({
  displayName: "TableHeaderCell__ResizeInner",
  componentId: "sc-1n0j3v0-0"
})(["width:2px;opacity:0;height:100%;"]);
const Resizer = styled.div.withConfig({
  displayName: "TableHeaderCell__Resizer",
  componentId: "sc-1n0j3v0-1"
})(["transform:", ";", "{opacity:", ";}position:absolute;right:0;top:0;height:100%;width:5px;cursor:col-resize;user-select:none;touch-action:none;display:flex;justify-content:flex-end;"], props => props.$columnResizeMode === 'onEnd' ? 'translateX(0px)' : 'none', ResizeInner, props => props.$isResizing ? 1 : 0);
const Cell = styled(Table.Cell).withConfig({
  displayName: "TableHeaderCell__Cell",
  componentId: "sc-1n0j3v0-2"
})(["font-weight:bold;height:30px;position:", ";top:0;", " ", ";&:hover ", "{background:", ";opacity:1;}"], p => p.$sticky || p.$pinned ? 'sticky' : 'relative', p => {
  if (p.$pinned) {
    return `${p.$pinned}: ${p.$offset}px;`;
  }
  return '';
}, p => {
  if (p.$sticky && p.$pinned) return 'z-index: 13';
  if (p.$sticky || p.$pinned) return 'z-index: 12';
}, ResizeInner, tokens.colors.interactive.primary__hover.rgba);
function TableHeaderCell({
  header,
  columnResizeMode
}) {
  const ctx = useTableContext();
  const table = ctx.table;
  const pinned = header.column.getIsPinned();
  const offset = useMemo(() => {
    if (!pinned) {
      return null;
    }
    return pinned === 'left' ? header.getStart() : table.getTotalSize() - header.getStart() - header.getSize();
  }, [pinned, header, table]);
  return header.isPlaceholder ? /*#__PURE__*/jsx(Cell, {
    $sticky: ctx.stickyHeader,
    $offset: offset,
    $pinned: pinned,
    className: ctx.headerClass ? ctx.headerClass(header.column) : '',
    style: {
      ...(ctx.headerStyle ? ctx.headerStyle(header.column) : {})
    },
    "aria-hidden": true
  }) : /*#__PURE__*/jsxs(Cell, {
    $sticky: ctx.stickyHeader,
    $offset: offset,
    $pinned: pinned,
    className: ctx.headerClass ? ctx.headerClass(header.column) : '',
    "aria-sort": getSortLabel(header.column.getIsSorted()),
    onClick: header.column.getToggleSortingHandler(),
    key: header.id,
    colSpan: header.colSpan,
    style: {
      width: header.getSize(),
      verticalAlign: ctx.enableColumnFiltering ? 'top' : 'middle',
      ...(ctx.headerStyle ? ctx.headerStyle(header.column) : {})
    },
    children: [/*#__PURE__*/jsxs(Fragment, {
      children: [/*#__PURE__*/jsx("div", {
        style: {
          display: 'flex',
          flexDirection: 'column'
        },
        children: /*#__PURE__*/jsx("span", {
          className: "table-header-cell-label",
          children: flexRender(header.column.columnDef.header, header.getContext())
        })
      }), {
        asc: /*#__PURE__*/jsx(Icon, {
          data: arrow_up
        }),
        desc: /*#__PURE__*/jsx(Icon, {
          data: arrow_down
        })
      }[header.column.getIsSorted()] ?? null, header.column.getCanFilter() ?
      
      // Supressing this warning - div is not interactive, but prevents propagation of events to avoid unintended sorting
      // eslint-disable-next-line jsx-a11y/click-events-have-key-events,jsx-a11y/no-static-element-interactions
      jsx("div", {
        onClick: e => e.stopPropagation(),
        children: /*#__PURE__*/jsx(Filter, {
          column: header.column,
          table: table
        })
      }) : null]
    }), columnResizeMode && /*#__PURE__*/jsx(Resizer, {
      onClick: e => e.stopPropagation(),
      onMouseDown: header.getResizeHandler(),
      onTouchStart: header.getResizeHandler(),
      $isResizing: header.column.getIsResizing(),
      $columnResizeMode: columnResizeMode,
      className: 'resize-handle',
      "data-testid": 'resize-handle',
      children: /*#__PURE__*/jsx(ResizeInner, {})
    })]
  });
}

export { TableHeaderCell };
